@@ -92,34 +92,91 @@ struct Command {
 
 fn print_help() {
     print!("Input/output is hex unless toggled to decimal with 'x'
-?          This help
-<Enter>    Print current byte(s) and move forward to next set of byte(s)
-3d4        Move to byte number 3d4 and print from there
-+          Move 1 byte forward and print from there
--          Move 1 byte back and print from there
-+3d4       Move 3d4 bytes forward and print from there
--3d4       Move 3d4 bytes back and print from there
-$          Move to last byte and print it
-k          Delete (kill) byte at current index and print new line of byte(s)
-7dk        Move to byte 7d, delete that byte, and print from there.
-1d,72k     Move to byte 1d, delete bytes 1d - 72 inclusive, and print from there.
-i          Prompt you to write out bytes which will be inserted at current index
-72i        Move to byte number 72 and prompt you to enter bytes which will be
-             inserted there.
-12,3dp     Print bytes 12 - 3d inclusive, move to leftmost byte printed on the
-             last line.
-m          Toggle whether or not characters are printed after bytes
-n          Toggle whether or not byte numbers are printed before bytes
-p          Print current line of byte(s) (depending on 'W')
-s          Print state of all toggles and 'W'idth
-x          Toggle interpreting inputs and displaying output as hex or decimal
-w          Actually write changes to the file on disk
-W3d        Print a linebreak every 3d bytes [Default 0x10]
-q          quit
+?            This help
+<Enter>      Print current byte(s) and move forward to next set of byte(s)
+3d4          Move to byte number 3d4 and print from there
++            Move 1 byte forward and print from there
++++          Move 3 bytes forward and print from there
+-            Move 1 byte back and print from there
++3d4         Move 3d4 bytes forward and print from there
+-3d4         Move 3d4 bytes back and print from there
+$            Move to last byte and print it
+/deadbeef    If the bytes de ad be ef exist after the current index, move there
+               and print.
+k            Delete (kill) byte at current index and print new line of byte(s)
+7dk          Move to byte 7d, delete that byte, and print from there.
+1d,72k       Move to byte 1d, delete bytes 1d - 72 inclusive, and print from there.
+/deadbeef/k  If the bytes de ad be ef exist after the current index, move there,
+               delete those bytes, and print.
+i            Prompt you to write out bytes which will be inserted at current index
+72i          Move to byte number 72 and prompt you to enter bytes which will be
+               inserted there.
+/deadbeef/i  If the bytes de ad be ef exist after the current index, move there
+               and prompt you to enter bytes which will be inserted there.
+12,3dp       Print bytes 12 - 3d inclusive, move to leftmost byte printed on the
+               last line.
+m            Toggle whether or not characters are printed after bytes
+n            Toggle whether or not byte numbers are printed before bytes
+p            Print current line of byte(s) (depending on 'W')
+s            Print state of all toggles and 'W'idth
+x            Toggle interpreting inputs and displaying output as hex or decimal
+w            Actually write changes to the file on disk
+W3d          Print a linebreak every 3d bytes [Default 0x10]
+q            quit
 ");
 }
 
 
+fn string_from_bytes(bytes:&[u8]) -> String {
+    let mut to_return = "".to_owned();
+    for &byte in bytes {
+        to_return += &padded_byte(byte);
+    }
+    return to_return;
+}
+
+
+fn index_of_bytes(needle:&[u8], haystack:&[u8]) -> Option<usize> {
+    let needle_num_bytes = if needle.len() == 0 {
+        return None;
+    }
+    else {
+        needle.len()
+    };
+
+    for index in 0..haystack.len() {
+        let range = (index, index + needle_num_bytes - 1);
+
+        if bad_range(&haystack.to_vec(), range) {
+            return None;
+        }
+
+
+        if &haystack[range.0..=range.1] == needle {
+            return Some(index);
+        }
+        else {
+
+        }
+    }
+
+    None
+}
+
+
+fn bytes_from_string(nibbles_s:&str) -> Result<Vec<u8>, String> {
+    // TODO Allow general whitespace, not just literal spaces
+    let re_bytes = Regex::new(r"^ *([0-9a-fA-F][0-9a-fA-F] *)* *$").unwrap();
+    if re_bytes.is_match(&nibbles_s) {
+        let nibbles_v:Vec<String> = nibbles_s.replace(" ", "").chars().map(|x| x.to_string()).collect();
+        Ok(nibbles_v.chunks(2).map(|x| x.join("")).map(|x| u8::from_str_radix(&x, 16).unwrap()).collect())
+    }
+    else {
+        Err(format!("Couldn't interpret '{}' as a sequence of bytes", &nibbles_s))
+    }
+}
+
+
 fn read_bytes_from_user() -> Result<Vec<u8>, String> {
     print!("> ");
     io::stdout().flush().unwrap();
@@ -130,24 +187,40 @@ fn read_bytes_from_user() -> Result<Vec<u8>, String> {
         }
     };
 
-    // TODO Allow general whitespace, not just literal spaces
-    let re_bytes = Regex::new(r"^ *([0-9a-fA-F][0-9a-fA-F] *)* *$").unwrap();
-    if re_bytes.is_match(&input) {
-        let nibbles:Vec<String> = input.replace(" ", "").chars().map(|x| x.to_string()).collect();
-        Ok(nibbles.chunks(2).map(|x| x.join("")).map(|x| u8::from_str_radix(&x, 16).unwrap()).collect())
-
-    }
-    else {
-        Err(format!("Couldn't interpret '{}' as a sequence of bytes", &input))
-    }
+    bytes_from_string(&input)
 }
 
 
+// TODO: This should take a usize..=usize range object
 fn bad_range(bytes: &Vec<u8>, range: (usize, usize)) -> bool {
     bytes.len() == 0 || range.1 >= bytes.len()
 }
 
 
+/// Returns new index
+fn move_to(state:&mut State, index:usize) -> Result<usize, String> {
+    if state.empty() {
+        Err("Empty file".to_owned())
+    }
+    else {
+        let _max_index = match state.max_index() {
+            Ok(max) => max,
+            Err(error) => {
+                return Err(error);
+            },
+        };
+
+        if index > _max_index {
+            Err(format!("{} > {} = maximum index", index, _max_index))
+        }
+        else {
+            state.index = index;
+            Ok(index)
+        }
+    }
+}
+
+
 impl Command {
     fn bad_range(&self, all_bytes: &Vec<u8>) -> bool {
         bad_range(all_bytes, self.range)
@@ -159,8 +232,11 @@ impl Command {
         // created over and over
         // TODO Allow general whitespace, not just literal spaces
         let re_blank_line = Regex::new(r"^ *$").unwrap();
-        let re_plus = Regex::new(r"^ *\+ *$").unwrap();
-        let re_minus = Regex::new(r"^ *\- *$").unwrap();
+        let re_pluses = Regex::new(r"^ *(?P<pluses>\++) *$").unwrap();
+        let re_minuses = Regex::new(r"^ *(?P<minuses>\-+) *$").unwrap();
+        let re_search = Regex::new(r"^ */(?P<bytes>[0-9a-fA-F]+) *$").unwrap();
+        let re_search_kill = Regex::new(r"^ */(?P<bytes>[0-9a-fA-F]+)/k *$").unwrap();
+        let re_search_insert = Regex::new(r"^ */(?P<bytes>[0-9a-fA-F]+)/i *$").unwrap();
         let re_single_char_command = Regex::new(r"^ *(?P<command>[?mnpsxqwik]).*$").unwrap();
         let re_range = Regex::new(r"^ *(?P<begin>[0-9a-fA-F.$]+) *, *(?P<end>[0-9a-fA-F.$]+) *(?P<the_rest>.*) *$").unwrap();
         let re_specified_index = Regex::new(r"^ *(?P<index>[0-9A-Fa-f.$]+) *(?P<the_rest>.*) *$").unwrap();
@@ -170,9 +246,12 @@ impl Command {
 
         let is_blank_line          = re_blank_line.is_match(line);
         let is_single_char_command = re_single_char_command.is_match(line);
-        let is_plus                = re_plus.is_match(line);
-        let is_minus               = re_minus.is_match(line);
+        let is_pluses              = re_pluses.is_match(line);
+        let is_minuses             = re_minuses.is_match(line);
         let is_range               = re_range.is_match(line);
+        let is_search              = re_search.is_match(line);
+        let is_search_kill         = re_search_kill.is_match(line);
+        let is_search_insert       = re_search_insert.is_match(line);
         let is_specified_index     = re_specified_index.is_match(line);
         let is_offset_index        = re_offset_index.is_match(line);
         let is_width               = re_width.is_match(line);
@@ -183,11 +262,20 @@ impl Command {
         else if is_single_char_command {
             re_single_char_command
         }
-        else if is_plus {
-            re_plus
+        else if is_search {
+            re_search
         }
-        else if is_minus {
-            re_minus
+        else if is_search_insert {
+            re_search_insert
+        }
+        else if is_search_kill {
+            re_search_kill
+        }
+        else if is_pluses {
+            re_pluses
+        }
+        else if is_minuses {
+            re_minuses
         }
         else if is_range {
             re_range
@@ -207,20 +295,91 @@ impl Command {
 
         let caps = re.captures(line);
 
-        if is_plus {
+        if is_pluses {
+            let num_pluses = num_graphemes(caps.unwrap().name("pluses").unwrap().as_str());
             Ok(Command{
-                range: (0, 0),
+                range: (num_pluses, num_pluses),
                 command: 'G',
                 args: vec![],
             })
         }
-        else if is_minus {
+
+        else if is_search_insert {
+            match bytes_from_string(caps.unwrap().name("bytes").unwrap().as_str()) {
+                Ok(needle) => {
+                    if let Some(offset) = index_of_bytes(&needle, &state.all_bytes[state.index..]) {
+                        Ok(Command{
+                            range: (state.index + offset, state.index + offset),
+                            command: 'i',
+                            args: vec![],
+                        })
+                    }
+                    else {
+                        Err(format!("{} not found", string_from_bytes(&needle)))
+                    }
+                },
+                Err(error) => {
+                    Err(error)
+                }
+            }
+        }
+
+        else if is_search_kill {
+            match bytes_from_string(caps.unwrap().name("bytes").unwrap().as_str()) {
+                Ok(needle) => {
+                    let needle_num_bytes = if needle.len() == 0 {
+                        return Err("Searching for empty string".to_owned());
+                    }
+                    else {
+                        needle.len()
+                    };
+
+                    if let Some(offset) = index_of_bytes(&needle, &state.all_bytes[state.index..]) {
+                        Ok(Command{
+                            range: (state.index + offset, state.index + offset + needle_num_bytes - 1),
+                            command: 'k',
+                            args: vec![],
+                        })
+                    }
+                    else {
+                        Err(format!("{} not found", string_from_bytes(&needle)))
+                    }
+                },
+                Err(error) => {
+                    Err(error)
+                }
+            }
+        }
+
+        else if is_search {
+            match bytes_from_string(caps.unwrap().name("bytes").unwrap().as_str()) {
+                Ok(needle) => {
+                    if let Some(offset) = index_of_bytes(&needle, &state.all_bytes[state.index..]) {
+                        Ok(Command{
+                            range: (state.index + offset, state.index + offset),
+                            command: 'g',
+                            args: vec![],
+                        })
+                    }
+                    else {
+                        Err(format!("{} not found", string_from_bytes(&needle)))
+                    }
+                },
+                Err(error) => {
+                    Err(error)
+                }
+            }
+        }
+
+        else if is_minuses {
+            let num_minuses = num_graphemes(caps.unwrap().name("minuses").unwrap().as_str());
             Ok(Command{
-                range: (0, 0),
+                range: (num_minuses, num_minuses),
                 command: 'H',
                 args: vec![],
             })
         }
+
         else if is_single_char_command {
             let command = caps.unwrap().name("command").unwrap().as_str().chars().next().unwrap();
             if command == 'p' {
@@ -510,6 +669,17 @@ mod tests {
         assert_eq!(padded_byte(10), "0a");
     }
 
+    #[test]
+    fn test_index_of_bytes() {
+        let haystack = vec![0xde, 0xad, 0xbe, 0xef];
+        assert_eq!(index_of_bytes(&vec![], &haystack), None);
+        assert_eq!(index_of_bytes(&vec![0xad, 0xbe], &haystack), Some(1));
+        assert_eq!(index_of_bytes(&vec![0xad, 0xbe, 0xef], &haystack), Some(1));
+        assert_eq!(index_of_bytes(&vec![0xad, 0xbe, 0xef, 0xef], &haystack), None);
+        assert_eq!(index_of_bytes(&vec![0xde,], &haystack), Some(0));
+    }
+
+
     #[test]
     fn test_max_bytes_line() {
         let _1 = NonZeroUsize::new(1).unwrap();
@@ -817,6 +987,51 @@ impl State {
     }
 }
 
+/// Returns new index number
+fn minuses(state:&mut State, num_minuses:usize) -> Result<usize, String> {
+    if state.empty() {
+        Err("Empty file".to_owned())
+    }
+    else if state.index == 0 {
+        Err("already at 0th byte".to_owned())
+    }
+    else if state.index < num_minuses {
+        Err(format!("Going back {} bytes would take you beyond the 0th byte", num_minuses))
+    }
+    else {
+        state.index -= num_minuses;
+        print_bytes(&state, state.range());
+        Ok(state.index)
+    }
+}
+
+/// Returns new index number
+fn pluses(state:&mut State, num_pluses:usize) -> Result<usize, String> {
+    if state.empty() {
+        Err("Empty file".to_owned())
+    }
+    else {
+        match state.max_index() {
+            Ok(max) => {
+                if state.index == max {
+                    Err("already at last byte".to_owned())
+                }
+                else if state.index + num_pluses > max {
+                    Err(format!("Moving {} bytes would put you past last byte", num_pluses))
+                }
+                else {
+                    state.index += num_pluses;
+                    print_bytes(&state, state.range());
+                    Ok(state.index)
+                }
+            },
+            Err(error) => {
+                Err(error)
+            },
+        }
+    }
+}
+
 
 pub fn actual_runtime(filename: &str) -> i32 {
     let file = match File::open(filename) {
@@ -905,51 +1120,37 @@ pub fn actual_runtime(filename: &str) -> i32 {
 
                     /* Go to */
                     'g' => {
-                        if command.bad_range(&state.all_bytes) {
-                            println!("? (bad range)");
-                            continue;
-                        }
-                        state.index = command.range.1;
-                        print_bytes(&state, state.range());
-                    },
-
-                    /* + */
-                    'G' => {
-                        if state.empty() {
-                            println!("? (Empty file)");
-                            continue;
-                        }
-
-                        match state.max_index() {
-                            Ok(max) => {
-                                if state.index == max {
-                                    println!("? (already at last byte");
-                                }
-                                else if state.index > max {
-                                    println!("? (past last byte");
-                                }
-                                else {
-                                    state.index += 1;
-                                    print_bytes(&state, state.range());
-                                }
+                        match move_to(&mut state, command.range.0) {
+                            Ok(_) => {
+                                print_bytes(&state, state.range());
                             },
                             Err(error) => {
                                 println!("? ({})", error);
-                            },
+                            }
                         }
                     },
 
-                    /* - */
+                    /* +'s */
+                    'G' => {
+                        match pluses(&mut state, command.range.0) {
+                            Err(error) => {
+                                println!("? ({})", error);
+                            },
+                            Ok(_) => {
+                                continue;
+                            }
+                        }
+                    },
+
+                    /* -'s */
                     'H' => {
-                        if state.empty() {
-                            println!("? (Empty file)");
-                        }
-                        else if state.index == 0 {
-                            println!("? (already at 0th byte");
-                        }
-                        else {
-                            state.index -= 1;
-                            print_bytes(&state, state.range());
+                        match minuses(&mut state, command.range.0) {
+                            Err(error) => {
+                                println!("? ({})", error);
+                            },
+                            Ok(_) => {
+                                continue;
+                            }
                         }
                     },
 
